/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc;

import net.minecraft.block.Block;
import net.minecraft.block.BlockLeaves;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.*;
import net.minecraft.util.EnumActionResult;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.event.world.BlockEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;

import net.dries007.tfc.api.capability.size.CapabilityItemSize;
import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.objects.blocks.BlockCharcoalPile;
import net.dries007.tfc.objects.blocks.BlocksTFC;
import net.dries007.tfc.objects.te.TELogPile;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.IFireable;
import net.dries007.tfc.util.IPlacableItem;
import net.dries007.tfc.util.OreDictionaryHelper;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;
import static net.dries007.tfc.objects.blocks.BlockCharcoalPile.LAYERS;

@Mod.EventBusSubscriber(modid = MOD_ID)
public class CommonEventHandler
{
    /**
     * Make leaves drop sticks
     */
    @SubscribeEvent
    public void onBlockHarvestDrops(BlockEvent.HarvestDropsEvent event)
    {
        final EntityPlayer harvester = event.getHarvester();
        final ItemStack heldItem = harvester == null ? ItemStack.field_190927_a : harvester.func_184614_ca();
        final IBlockState state = event.getState();
        final Block block = state.func_177230_c();

        if (!event.isSilkTouching() && block instanceof BlockLeaves)
        {
            // Done via event so it applies to all leaves.
            double chance = ConfigTFC.GENERAL.leafStickDropChance;
            if (!heldItem.func_190926_b() && Helpers.containsAnyOfCaseInsensitive(heldItem.func_77973_b().getToolClasses(heldItem), ConfigTFC.GENERAL.leafStickDropChanceBonusClasses))
                chance = ConfigTFC.GENERAL.leafStickDropChanceBonus;
            if (event.getWorld().field_73012_v.nextFloat() < chance)
                event.getDrops().add(new ItemStack(Items.field_151055_y));
        }
    }

    /**
     * Place pit kiln block & add items
     * Note: `onBlockActivate` doesn't get called when the player is sneaking, unless doesSneakBypassUse returns true.
     * We have this event already, might as well use it.
     */
    @SubscribeEvent
    public void onRightClickBlock(PlayerInteractEvent.RightClickBlock event)
    {
        World world = event.getWorld();
        BlockPos pos = event.getPos();
        ItemStack stack = event.getItemStack();
        EntityPlayer player = event.getEntityPlayer();

        /*
         Note: This event handler is fired first with the main hand as event.getStack()
         If nothing happens (as per vanilla behavior, even if this event causes something to happen),
         The event will fire AGAIN with the offhand and offhand stack.

         This is to prevent that second firing of the event from causing duplicate actions
         i.e. if you hold charcoal with main hand and a block of dirt with offhand, the dirt will try and place because
         vanilla behavior doesn't do anything with charcoal in the main hand
        */
        if (event.getHand() == EnumHand.OFF_HAND)
        {
            ItemStack mainStack = player.func_184586_b(EnumHand.MAIN_HAND);
            if ((mainStack.func_77973_b() == Items.field_151044_h && mainStack.func_77960_j() == 1) ||
                (OreDictionaryHelper.doesStackMatchOre(mainStack, "logWood") && player.func_70093_af()) ||
                mainStack.func_77973_b() instanceof IPlacableItem)
            {
                event.setCanceled(true);
                return;
            }

            IFireable fireable = IFireable.fromItem(event.getItemStack().func_77973_b());
            if (fireable != null && event.getEntityPlayer().func_70093_af() && event.getFace() == EnumFacing.UP)
            {
                event.setCanceled(true);
                return;
            }
        }

        if (stack.func_77973_b() == Items.field_151044_h && stack.func_77960_j() == 1)
        {
            EnumFacing facing = event.getFace();
            if (facing != null)
            {
                if (world.func_180495_p(pos.func_177977_b().func_177972_a(facing)).func_185915_l()
                    && world.func_180495_p(pos.func_177972_a(facing)).func_177230_c().func_176200_f(world, pos.func_177972_a(facing)))
                {

                    if (world.func_180495_p(pos).func_177230_c() instanceof BlockCharcoalPile)
                    {
                        if (world.func_180495_p(pos).func_177229_b(LAYERS) != 8)
                        {
                            // Adding layers is handled in BlockCharcoalPile
                            return;
                        }
                    }
                    if (!world.field_72995_K)
                    {
                        // noinspection ConstantConditions
                        world.func_175656_a(pos.func_177972_a(facing), BlocksTFC.CHARCOAL_PILE.func_176223_P());

                        if (!player.func_184812_l_())
                        {
                            player.func_184611_a(event.getHand(), Helpers.consumeItem(stack, 1));
                        }
                        world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187587_bZ, SoundCategory.BLOCKS, 1.0F, 0.5F);
                        return;
                    }

                }
            }
        }
        if (OreDictionaryHelper.doesStackMatchOre(stack, "logWood") && player.func_70093_af())
        {
            EnumFacing facing = event.getFace();
            if (facing != null)
            {
                //noinspection ConstantConditions
                if (world.func_180495_p(pos).func_177230_c() == BlocksTFC.LOG_PILE)
                {
                    if (!world.field_72995_K)
                    {
                        TELogPile te = Helpers.getTE(world, pos, TELogPile.class);
                        if (te != null)
                        {
                            if (te.insertLog(stack.func_77946_l()))
                            {
                                player.func_184611_a(event.getHand(), Helpers.consumeItem(stack, player, 1));
                                world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187891_gV, SoundCategory.BLOCKS, 1.0F, 1.0F);
                            }
                            else
                            {
                                // Insert log didn't work, see if trying to place another log pile
                                if (facing == EnumFacing.UP && te.countLogs() == 16 || (facing != EnumFacing.UP && world.func_180495_p(pos.func_177977_b().func_177972_a(facing)).func_185915_l()
                                    && world.func_180495_p(pos.func_177972_a(facing)).func_177230_c().func_176200_f(world, pos.func_177972_a(facing))))
                                {
                                    // noinspection ConstantConditions
                                    world.func_175656_a(pos.func_177972_a(facing), BlocksTFC.LOG_PILE.func_180642_a(world, pos, facing, 0, 0, 0, 0, player));

                                    TELogPile te2 = Helpers.getTE(world, pos.func_177972_a(facing), TELogPile.class);
                                    if (te2 != null)
                                    {
                                        te2.insertLog(stack.func_77946_l());
                                    }

                                    player.func_184611_a(event.getHand(), Helpers.consumeItem(stack, player, 1));
                                    world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187891_gV, SoundCategory.BLOCKS, 1.0F, 1.0F);
                                }
                            }
                        }
                    }
                }
                else
                {
                    if (world.func_180495_p(pos.func_177977_b().func_177972_a(facing)).func_185915_l()
                        && world.func_180495_p(pos.func_177972_a(facing)).func_177230_c().func_176200_f(world, pos.func_177972_a(facing)) &&
                        player.func_70093_af())
                    {
                        // Place log pile
                        if (!world.field_72995_K)
                        {
                            // noinspection ConstantConditions
                            world.func_175656_a(pos.func_177972_a(facing), BlocksTFC.LOG_PILE.func_180642_a(world, pos, facing, 0, 0, 0, 0, player));

                            TELogPile te = Helpers.getTE(world, pos.func_177972_a(facing), TELogPile.class);
                            if (te != null)
                            {
                                te.insertLog(stack.func_77946_l());
                            }

                            player.func_184611_a(event.getHand(), Helpers.consumeItem(stack, player, 1));
                            world.func_184133_a(null, pos.func_177972_a(facing), SoundEvents.field_187891_gV, SoundCategory.BLOCKS, 1.0F, 1.0F);
                        }
                    }
                }
            }
            event.setCancellationResult(EnumActionResult.SUCCESS);
            event.setCanceled(true);
            return;
        }

        // All items that can should use this implementation instead of the other exceptions (which are for items that can't implement IPlacableItem)
        if (stack.func_77973_b() instanceof IPlacableItem)
        {
            IPlacableItem item = (IPlacableItem) stack.func_77973_b();
            if (item.placeItemInWorld(world, pos, stack, player, event.getFace(), event.getHitVec()))
            {
                player.func_184611_a(event.getHand(), Helpers.consumeItem(stack, player, 1));

                event.setCancellationResult(EnumActionResult.SUCCESS);
                event.setCanceled(true);
            }
        }
    }

    //Used for IItemSize capability. You can either implement the interface or use the capability
    @SubscribeEvent
    public void attachItemCapabilities(AttachCapabilitiesEvent<ItemStack> e)
    {
        ItemStack stack = e.getObject();
        // Skip items with existing capabilities
        if (CapabilityItemSize.getIItemSize(stack) != null) return;

        Item item = stack.func_77973_b();
        boolean canStack = stack.func_77976_d() > 1; // This is nessecary so it isn't accidentally overriden by a default implementation

        // todo: Add more items here
        if (item == Items.field_151044_h)
            CapabilityItemSize.add(e, Items.field_151044_h, Size.SMALL, Weight.MEDIUM, canStack);
        else if (item == Items.field_151055_y)
            CapabilityItemSize.add(e, Items.field_151055_y, Size.SMALL, Weight.LIGHT, canStack);

            // Final checks for general item types
        else if (item instanceof ItemTool)
            CapabilityItemSize.add(e, item, Size.LARGE, Weight.MEDIUM, canStack);
        else if (item instanceof ItemArmor)
            CapabilityItemSize.add(e, item, Size.LARGE, Weight.HEAVY, canStack);
        else if (item instanceof ItemBlock)
            CapabilityItemSize.add(e, item, Size.SMALL, Weight.MEDIUM, canStack);
        else
            CapabilityItemSize.add(e, item, Size.VERY_SMALL, Weight.LIGHT, canStack);
    }
}
