/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.client.render;

import net.minecraft.client.Minecraft;
import net.minecraft.client.model.ModelBase;
import net.minecraft.client.model.ModelRenderer;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.renderer.RenderHelper;
import net.minecraft.client.renderer.RenderItem;
import net.minecraft.client.renderer.block.model.ItemCameraTransforms;
import net.minecraft.client.renderer.tileentity.TileEntitySpecialRenderer;
import net.minecraft.entity.Entity;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraft.util.ResourceLocation;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.objects.te.TEPitKiln;

import static net.dries007.tfc.api.util.TFCConstants.MOD_ID;

@SideOnly(Side.CLIENT)
public class TESRPitKiln extends TileEntitySpecialRenderer<TEPitKiln>
{
    private static final ResourceLocation THATCH = new ResourceLocation(MOD_ID, "textures/blocks/thatch.png");
    private static final ResourceLocation BARK = new ResourceLocation(MOD_ID, "textures/blocks/wood/log/oak.png");
    private static final ModelHay[] HAY = new ModelHay[TEPitKiln.STRAW_NEEDED];
    private static final int LOG_ROWS = 2;
    private static final int LOGS_PER_ROW = TEPitKiln.WOOD_NEEDED / LOG_ROWS;
    private static final ModelLog LOG = new ModelLog();
    private static final float SCALE = 1f / 16f;

    static
    {
        for (int i = 0; i < TEPitKiln.STRAW_NEEDED; i++)
        {
            HAY[i] = new ModelHay(i);
        }
    }

    @Override
    public void func_192841_a(TEPitKiln te, double x, double y, double z, float partialTicks, int destroyStage, float alpha)
    {
        RenderItem renderItem = Minecraft.func_71410_x().func_175599_af();
        World world = te.func_145831_w();
        //noinspection ConstantConditions
        if (world == null) return;

        GlStateManager.func_179094_E();
        GlStateManager.func_179123_a();
        GlStateManager.func_179140_f();
        GlStateManager.func_179137_b(x, y, z);

        GlStateManager.func_179094_E();
        NonNullList<ItemStack> items = te.getItems();
        float timeD = (float) (360.0 * (System.currentTimeMillis() & 0x3FFFL) / 0x3FFFL);
        GlStateManager.func_179152_a(0.5F, 0.5F, 0.5F);
        GlStateManager.func_179137_b(0.5, 0.5, 0.5);
        RenderHelper.func_74519_b();
        GlStateManager.func_179123_a();
        for (int i = 0; i < items.size(); i++)
        {
            ItemStack stack = items.get(i);
            if (stack.func_190926_b()) continue;
            GlStateManager.func_179094_E();
            GlStateManager.func_179109_b((i % 2 == 0 ? 1 : 0), 0, (i < 2 ? 1 : 0));
            GlStateManager.func_179114_b(timeD, 0, 1, 0);
            renderItem.func_181564_a(stack, ItemCameraTransforms.TransformType.FIXED);
            GlStateManager.func_179121_F();
        }
        RenderHelper.func_74518_a();
        GlStateManager.func_179099_b();
        GlStateManager.func_179121_F();

        int straw = te.getStrawCount();
        if (straw != 0)
        {
            GlStateManager.func_179094_E();
            GlStateManager.func_179123_a();
            GlStateManager.func_179145_e();
            GlStateManager.func_179091_B();

            func_147499_a(THATCH);
            HAY[straw - 1].func_78088_a(null, 0, 0, 0, 0, 0, SCALE);

            GlStateManager.func_179099_b();
            GlStateManager.func_179121_F();
        }

        int logs = te.getLogCount();
        if (logs != 0)
        {
            GlStateManager.func_179094_E();
            GlStateManager.func_179123_a();
            GlStateManager.func_179145_e();
            GlStateManager.func_179091_B();
            GlStateManager.func_179137_b(0, 0.5d, 0);

            func_147499_a(BARK);

            for (int row = 0; row < LOG_ROWS && logs > 0; row++)
            {
                GlStateManager.func_179094_E();
                GlStateManager.func_179137_b(0, row * 0.5d / (double) LOG_ROWS, 0);
                for (int i = 0; i < LOGS_PER_ROW && logs > 0; i++, logs--)
                {
                    GlStateManager.func_179094_E();
                    GlStateManager.func_179137_b(0, 0, i / (double) LOGS_PER_ROW);
                    LOG.func_78088_a(null, 0, 0, 0, 0, 0, SCALE);
                    GlStateManager.func_179121_F();
                }
                GlStateManager.func_179121_F();
            }
            GlStateManager.func_179099_b();
            GlStateManager.func_179121_F();
        }

        GlStateManager.func_179099_b();
        GlStateManager.func_179121_F();
    }

    private static class ModelHay extends ModelBase
    {
        private final ModelRenderer hayRenderer;

        public ModelHay(int height)
        {
            field_78089_u = 16;
            field_78090_t = 16;
            hayRenderer = new ModelRenderer(this, 0, 0);
            hayRenderer.func_78789_a(0, 0, 0, 16, height + 1, 16);
        }

        @Override
        public void func_78088_a(Entity entityIn, float limbSwing, float limbSwingAmount, float ageInTicks, float netHeadYaw, float headPitch, float scale)
        {
            hayRenderer.func_78785_a(scale);
        }
    }

    private static class ModelLog extends ModelBase
    {
        private final ModelRenderer logRenderer;

        public ModelLog()
        {
            field_78089_u = 16;
            field_78090_t = 16;
            logRenderer = new ModelRenderer(this, 0, 0);
            logRenderer.func_78789_a(0, 0, 0, 16, 8 / LOG_ROWS, 16 / LOGS_PER_ROW);
        }

        @Override
        public void func_78088_a(Entity entityIn, float limbSwing, float limbSwingAmount, float ageInTicks, float netHeadYaw, float headPitch, float scale)
        {
            logRenderer.func_78785_a(scale);
        }
    }
}
