/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks;

import java.util.Random;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.Block;
import net.minecraft.block.ITileEntityProvider;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Items;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.*;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import net.dries007.tfc.client.TFCGuiHandler;
import net.dries007.tfc.objects.items.ItemFireStarter;
import net.dries007.tfc.objects.te.TEFirePit;

@ParametersAreNonnullByDefault
public class BlockFirePit extends Block implements ITileEntityProvider
{
    public static final PropertyBool LIT = PropertyBool.func_177716_a("lit");
    private static final AxisAlignedBB AABB = new AxisAlignedBB(0.0625D, 0.0D, 0.0625D, 0.9375D, 0.03125D, 0.9375D);

    public BlockFirePit()
    {
        super(Material.field_151581_o);
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(LIT, false));
        func_149649_H();
        func_149675_a(true);
        func_149715_a(1F);
    }

    @Override
    @SuppressWarnings("deprecation")
    @Nonnull
    public IBlockState func_176203_a(int meta)
    {
        return this.func_176223_P().func_177226_a(LIT, meta == 1);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(LIT) ? 1 : 0;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149686_d(IBlockState state)
    {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        return AABB;
    }

    @Override
    @Nonnull
    @SuppressWarnings("deprecation")
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face)
    {
        return BlockFaceShape.UNDEFINED;
    }

    @Override
    @SuppressWarnings("deprecation")
    public boolean func_149662_c(IBlockState state)
    {
        return false;
    }

    @Override
    public void func_180650_b(World worldIn, BlockPos pos, IBlockState state, Random rand)
    {
        if (worldIn.func_175727_C(pos)) //todo
            worldIn.func_180501_a(pos, state.func_177226_a(LIT, false), 2);
    }

    @SideOnly(Side.CLIENT)
    @Override
    public void func_180655_c(IBlockState state, World world, BlockPos pos, Random rng)
    {
        if (!state.func_177229_b(LIT)) return;

        if (rng.nextInt(24) == 0)
        {
            world.func_184134_a((double) ((float) pos.func_177958_n() + 0.5F), (double) ((float) pos.func_177956_o() + 0.5F), (double) ((float) pos.func_177952_p() + 0.5F), SoundEvents.field_187643_bs, SoundCategory.BLOCKS, 1.0F + rng.nextFloat(), rng.nextFloat() * 0.7F + 0.3F, false);
        }
        double x = pos.func_177958_n() + 0.5;
        double y = pos.func_177956_o() + 0.1;
        double z = pos.func_177952_p() + 0.5;

        //todo: vary speed
        world.func_175688_a(EnumParticleTypes.SMOKE_NORMAL, x + rng.nextFloat() - 0.5, y, z + rng.nextFloat() - 0.5, 0.0D, 0.2D, 0.0D);
        if (rng.nextFloat() > 0.75)
            world.func_175688_a(EnumParticleTypes.SMOKE_LARGE, x + rng.nextFloat() - 0.5, y, z + rng.nextFloat() - 0.5, 0.0D, 0.1D, 0.0D);
    }

    @Override
    @SuppressWarnings("deprecation")
    public void func_189540_a(IBlockState state, World worldIn, BlockPos pos, Block blockIn, BlockPos fromPos)
    {
        if (!canBePlacedOn(worldIn, pos.func_177982_a(0, -1, 0)))
            worldIn.func_175698_g(pos);
    }

    @Override
    @Nonnull
    public Item func_180660_a(IBlockState state, Random rand, int fortune)
    {
        return Items.field_190931_a;
    }

    @Override
    @Nonnull
    @SideOnly(Side.CLIENT)
    public BlockRenderLayer func_180664_k()
    {
        return BlockRenderLayer.CUTOUT;
    }

    @Override
    public boolean func_176196_c(World worldIn, BlockPos pos)
    {
        return super.func_176196_c(worldIn, pos) && canBePlacedOn(worldIn, pos.func_177982_a(0, -1, 0));
    }

    // todo: override the fire stuff, see BlockPitKiln

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer player, EnumHand hand, EnumFacing facing, float hitX, float hitY, float hitZ)
    {
        if (!worldIn.field_72995_K)
        {
            ItemStack held = player.func_184586_b(hand);
            if (state.func_177229_b(LIT)) //todo: remove debug feature
                worldIn.func_175656_a(pos, state.func_177226_a(LIT, false));

            if (ItemFireStarter.canIgnite(held))
                worldIn.func_175656_a(pos, state.func_177226_a(LIT, true));

            if (!player.func_70093_af())
            {
                TFCGuiHandler.openGui(worldIn, pos, player, TFCGuiHandler.Type.FIRE_PIT);
            }

        }
        //todo: gui
        return true;
    }

    @Override
    @Nonnull
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, LIT);
    }

    @Override
    public int getLightValue(IBlockState state, IBlockAccess world, BlockPos pos)
    {
        return state.func_177229_b(LIT) ? super.getLightValue(state, world, pos) : 0;
    }

    @Override
    public boolean isBurning(IBlockAccess world, BlockPos pos)
    {
        return world.func_180495_p(pos).func_177229_b(LIT);
    }

    @Nullable
    @Override
    public TileEntity func_149915_a(World worldIn, int meta)
    {
        return new TEFirePit();
    }

    private boolean canBePlacedOn(World worldIn, BlockPos pos)
    {
        return worldIn.func_180495_p(pos).isSideSolid(worldIn, pos, EnumFacing.UP);
    }
}
