/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.objects.blocks.wood;

import java.util.*;
import javax.annotation.Nullable;
import javax.annotation.ParametersAreNonnullByDefault;

import net.minecraft.block.BlockLog;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.state.BlockStateContainer;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.init.Items;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.Explosion;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import mcp.MethodsReturnNonnullByDefault;
import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.api.capability.size.IItemSize;
import net.dries007.tfc.api.capability.size.Size;
import net.dries007.tfc.api.capability.size.Weight;
import net.dries007.tfc.api.types.Tree;
import net.dries007.tfc.util.Helpers;
import net.dries007.tfc.util.OreDictionaryHelper;

import net.minecraft.block.BlockLog.EnumAxis;

@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public class BlockLogTFC extends BlockLog implements IItemSize
{
    public static final PropertyBool PLACED = PropertyBool.func_177716_a("placed");
    public static final PropertyBool SMALL = PropertyBool.func_177716_a("small");
    public static final AxisAlignedBB SMALL_AABB_Y = new AxisAlignedBB(0.25, 0, 0.25, 0.75, 1, 0.75);
    public static final AxisAlignedBB SMALL_AABB_X = new AxisAlignedBB(0, 0.25, 0.25, 1, 0.75, 0.75);
    public static final AxisAlignedBB SMALL_AABB_Z = new AxisAlignedBB(0.25, 0.25, 0, 0.75, 0.75, 1);
    private static final Map<Tree, BlockLogTFC> MAP = new HashMap<>();

    public static BlockLogTFC get(Tree wood)
    {
        return MAP.get(wood);
    }

    public final Tree wood;

    public BlockLogTFC(Tree wood)
    {
        this.wood = wood;
        if (MAP.put(wood, this) != null) throw new IllegalStateException("There can only be one.");
        func_180632_j(field_176227_L.func_177621_b().func_177226_a(field_176299_a, BlockLog.EnumAxis.Y).func_177226_a(PLACED, true).func_177226_a(SMALL, false));
        setHarvestLevel("axe", 0);
        func_149711_c(15.0F);
        func_149752_b(5.0F);
        OreDictionaryHelper.register(this, "log", "wood");
        OreDictionaryHelper.register(this, "log", "wood", wood.getRegistryName().func_110623_a());
        Blocks.field_150480_ab.func_180686_a(this, 5, 5);
        func_149675_a(true);
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state)
    {
        return !state.func_177229_b(SMALL);
    }

    @Override
    @SuppressWarnings("deprecation")
    public float func_176195_g(IBlockState blockState, World worldIn, BlockPos pos)
    {
        return (blockState.func_177229_b(PLACED) ? 1.0f : 2.5f) * super.func_176195_g(blockState, worldIn, pos);
    }

    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess source, BlockPos pos)
    {
        if (!state.func_177229_b(SMALL)) return field_185505_j;
        switch (state.func_177229_b(field_176299_a))
        {
            case X:
                return SMALL_AABB_X;
            case Y:
                return SMALL_AABB_Y;
            case Z:
                return SMALL_AABB_Z;
        }
        return field_185505_j;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state)
    {
        return !state.func_177229_b(SMALL);
    }

    @Override
    public void func_180650_b(World world, BlockPos pos, IBlockState state, Random rand)
    {
        // For floating tree things, just make them gently disappear over time
        if (state.func_177229_b(PLACED)) return;
        for (int x = -1; x <= 1; x++)
            for (int y = -1; y <= 1; y++)
                for (int z = -1; z <= 1; z++)
                    if (world.func_180495_p(pos.func_177982_a(x, y, z)).func_177230_c() == this && (z != 0 || y != 0 || x != 0))
                        return;
        world.func_175698_g(pos);
    }

    @Override
    public void func_180652_a(World worldIn, BlockPos pos, Explosion explosionIn)
    {
        if (worldIn.field_72995_K) return;
        removeTree(worldIn, pos, null, ItemStack.field_190927_a, false);
    }

    @Override
    public void func_176208_a(World world, BlockPos pos, IBlockState state, EntityPlayer player)
    {
        if (world.field_72995_K || state.func_177229_b(PLACED)) return;
        final ItemStack stack = player.func_184614_ca();
        final Set<String> toolClasses = stack.func_77973_b().getToolClasses(stack);
        if (toolClasses.contains("axe"))
        {
            removeTree(world, pos, player, stack,
                OreDictionaryHelper.doesStackMatchOre(stack, "axeStone") ||
                    OreDictionaryHelper.doesStackMatchOre(stack, "hammerStone")
            );
        }
        else if (toolClasses.contains("hammer")) //
        {
            // Break log and spawn some sticks
            world.func_175698_g(pos);
            Helpers.spawnItemStack(world, pos.func_177963_a(0.5D, 0.5D, 0.5D), new ItemStack(Items.field_151055_y, 1 + (int) (Math.random() * 3)));
        }
    }

    @Override
    public IBlockState func_176203_a(int meta)
    {
        return func_176223_P().func_177226_a(field_176299_a, EnumAxis.values()[meta & 0b11]).func_177226_a(PLACED, (meta & 0b100) == 0b100).func_177226_a(SMALL, (meta & 0b1000) == 0b1000);
    }

    @Override
    public int func_176201_c(IBlockState state)
    {
        return state.func_177229_b(field_176299_a).ordinal() | (state.func_177229_b(PLACED) ? 0b100 : 0) | (state.func_177229_b(SMALL) ? 0b1000 : 0);
    }

    @Override
    protected BlockStateContainer func_180661_e()
    {
        return new BlockStateContainer(this, field_176299_a, PLACED, SMALL);
    }

    @Override
    public void func_180663_b(World worldIn, BlockPos pos, IBlockState state)
    {
        // Don't do vanilla leaf decay
    }

    @Override
    public IBlockState func_180642_a(World worldIn, BlockPos pos, EnumFacing facing, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer)
    {
        return super.func_180642_a(worldIn, pos, facing, hitX, hitY, hitZ, meta, placer).func_177226_a(PLACED, true).func_177226_a(SMALL, placer.func_70093_af());
    }

    private boolean removeTree(World world, BlockPos pos, @Nullable EntityPlayer player, ItemStack stack, boolean stoneTool)
    {
        final boolean explosion = stack.func_190926_b() || player == null;
        final int maxLogs = explosion ? Integer.MAX_VALUE : 1 + stack.func_77958_k() - stack.func_77952_i();

        // Find all logs and add them to a list
        List<BlockPos> logs = new ArrayList<>(50);
        List<BlockPos> checked = new ArrayList<>(50 * 3 * 3);
        logs.add(pos);
        for (int i = 0; i < logs.size(); i++)
        {
            final BlockPos pos1 = logs.get(i);
            // check for nearby logs
            for (int x = -1; x <= 1; x++)
            {
                for (int y = -1; y <= 1; y++)
                {
                    for (int z = -1; z <= 1; z++)
                    {
                        final BlockPos pos2 = pos1.func_177982_a(x, y, z);
                        if (checked.contains(pos2)) continue;
                        checked.add(pos2);
                        IBlockState state = world.func_180495_p(pos2);
                        if (state.func_177230_c() == this && !state.func_177229_b(PLACED))
                            logs.add(pos2);
                    }
                }
            }
        }
        // Sort the list in terms of max distance to the original tree
        logs.sort(Comparator.comparing(x -> x.func_177951_i(pos)));

        // Start removing logs*/
        for (final BlockPos pos1 : logs.subList(0, Math.min(logs.size(), maxLogs)))
        {
            if (explosion)
            {
                // Explosions are 30% Efficient: no TNT powered tree farms.
                if (Math.random() < 0.3)
                    Helpers.spawnItemStack(world, pos.func_177963_a(0.5d, 0.5d, 0.5d), new ItemStack(Item.func_150898_a(this)));
            }
            else
            {
                // Stone tools are 60% efficient
                if (!stoneTool || Math.random() < 0.6)
                    func_180657_a(world, player, pos1, world.func_180495_p(pos1), null, stack);
                stack.func_77972_a(1, player);
            }
            world.func_175698_g(pos1);
        }
        return maxLogs >= logs.size();
    }

    @Override
    public Size getSize(ItemStack stack)
    {
        return Size.NORMAL;
    }

    @Override
    public Weight getWeight(ItemStack stack)
    {
        return Weight.MEDIUM;
    }

}
